@@ -14,6 +14,7 @@ from .gui import gui
 from .validator import validate
 from .spec import Spec
 from .docs import generate_docs
+from .version import VERSION
 
 
 def setup_logging() -> logging.Logger:
@@ -37,7 +38,7 @@ def setup_logging() -> logging.Logger:
 
 
 def get_spec(json_file: str, logger: logging.Logger) -> Spec:
-    """ Create Spec from json file path.
+    """Create Spec from json file path.
     :param json_file: path to the json file.
     :param logger: logger.
     :return: Spec.
@@ -116,7 +117,7 @@ def write_dbc_cmd(json_file: str, dbc: str):
 @click.argument("skel")
 @click.option("--noformat", is_flag=True, default=False)
 def c_gen_cmd(templates: str, output: str, json_file: str, skel: str, noformat: bool):
-    """ Transform FCP json into a C library.
+    """Transform FCP json into a C library.
     :param templates: jinja template directory.
     :param output: output directory.
     :param json_file: FCP json file path.
@@ -138,7 +139,7 @@ def c_gen_cmd(templates: str, output: str, json_file: str, skel: str, noformat:
 @click.command(name="init")
 @click.argument("json_file")
 def init_cmd(json_file: str):
-    """ Create a basic FCP json file. 
+    """Create a basic FCP json file. 
     :param json_file: FCP json file path.
     """
 
@@ -149,7 +150,7 @@ def init_cmd(json_file: str):
 @click.command(name="validate")
 @click.argument("json_file")
 def validate_cmd(json_file: str):
-    """ Verify correctness of FCP json file.
+    """Verify correctness of FCP json file.
     :param json_file: FCP json file path.
     """
 
@@ -159,15 +160,15 @@ def validate_cmd(json_file: str):
 
     r, msg = validate(logger, json.loads(j))
     if r:
-        print("JSON validated")
+        print("✓ JSON validated")
     else:
-        print(msg)
+        print("❌" + msg)
 
 
 @click.command(name="gui")
 @click.argument("file", required=False)
 def gui_cmd(file: str):
-    """ Launch FCP json editor GUI.
+    """Launch FCP json editor GUI.
     :param file: Optional FCP json file path
     """
     logger = setup_logging()
@@ -256,6 +257,19 @@ def dump_signal_list(json_file, message):
             for signal in msg.signals.values():
                 print(signal)
 
+@click.command()
+@click.argument("json_file")
+@click.argument("message", required=False)
+def dump_signal_names(json_file, message):
+    logger = setup_logging()
+    spec = get_spec(json_file, logger)
+
+    for dev in spec.devices.values():
+        for msg in dev.msgs.values():
+            if message != None and msg.name != message:
+                continue
+            for signal in msg.signals.values():
+                print(signal.name)
 
 @click.command()
 @click.argument("json_file")
@@ -326,12 +340,13 @@ def docs(json_file: str, out: str, link_location: str):
     spec = get_spec(json_file, logger)
     generate_docs(spec, out, link_location, logger)
 
-
 @click.group(invoke_without_command=True)
 @click.option("--version", is_flag=True, default=False)
 def main(version):
-    VERSION = "0.1"
-
+    """CLI utility for managment of FCP JSON files."""
+        
+    if len(sys.argv) == 1:
+        print("fcp cli util.\nVersion:", VERSION, "\nFor usage see fcp --help")
     if version:
         click.echo(VERSION)
     pass
@@ -349,6 +364,7 @@ main.add_command(dump_cfg_list)
 main.add_command(dump_cmd_list)
 main.add_command(dump_log_list)
 main.add_command(dump_signal_list)
+main.add_command(dump_signal_names)
 main.add_command(print_log)
 main.add_command(print_dev)
 main.add_command(print_msg)
@@ -357,3 +373,6 @@ main.add_command(docs)
 
 if __name__ == "__main__":
     main()
+
+
+
