@@ -1,14 +1,12 @@
 from enum import Enum
-
-# from dataclasses import dataclass, field
 from typing import *
-
 import copy
-from pprint import pprint
-
 import datetime
+#from ww import f
 
-from ww import f
+
+def handle_key_not_found(d: dict, key: str):
+    return d.get(key).items() if d.get(key) != None else []
 
 
 def field(default_factory):
@@ -133,10 +131,102 @@ class Log:
         return hash((self.name, self.id, self.creation_date))
 
     def __repr__(self):
-        return f(
-            "name: {self.name}, id: {self.id}, string: {self.string}, n_args: {self.n_args}, comment: {self.comment}"
-        )
+        return "name: {}, id: {}, string: {}, n_args: {}, comment: {}".format(self.name, self.id, self.string, self.n_args, self.comment)
 
+class EnumValue:
+    """ Fcp EnumValue. C lookalike for FCP type definitions with name-value
+    associations.
+    """
+
+    def __init__(self) -> None:
+        self.name = ""
+        self.value = 0
+        self.creation_date = datetime.datetime.now()
+
+    def compile(self) -> Dict[str, Any]:
+        d = copy.deepcopy(self.__dict__)
+        del d["creation_date"]
+        return d
+
+    def decompile(self, d: Dict[str, Any]) -> None:
+        """ Transform node dictionary representation into a python class.
+
+        :param d: Node dictionary
+        """
+        self.__dict__.update(d)
+    
+    def get_name(self) -> str:
+        return self.name
+
+    def set_name(self, name: str) -> None:
+        self.name = name
+
+    def get_value(self) -> int:
+        return self.value
+
+    def set_value(self, value: int) -> None:
+        self.value = value
+
+    def __hash__(self):
+        return hash((self.name, self.creation_date))
+
+    def __repr__(self):
+        return "name: {}".format(self.name)
+
+class Enum:
+    """ Fcp Enum. C lookalike for FCP type definitions with name-value
+    associations.
+    """
+
+    def __init__(self: Enum) -> None:
+        self.name = ""
+        self.enumeration = {}
+        self.creation_date = datetime.datetime.now()
+
+    def compile(self: Enum) -> Dict[str, Any]:
+        enums = {}
+
+        for k, v in self.enumeration.items():
+            enums[k] = v.compile()
+
+        d = copy.deepcopy(self.__dict__)
+        d["enumeration"] = enums
+        del d["creation_date"]
+        return d
+
+    def decompile(self, d: Dict[str, Any]) -> None:
+        """ Transform node dictionary representation into a python class.
+
+        :param d: Node dictionary
+        """
+        enumeration = d["enumeration"]
+        del d["enumeration"]
+        
+        self.__dict__.update(d)
+
+        for k,v in enumeration.items():
+            enum_value = EnumValue()
+            print(v)
+            enum_value.decompile(v)
+            self.enumeration[k] = enum_value
+    
+    def get_name(self) -> str:
+        return self.name
+
+    def set_name(self, name: str) -> None:
+        self.name = name
+
+    def get_value(self) -> int:
+        return int(self.value)
+
+    def set_value(self, name: int) -> None:
+        self.value = value
+
+    def __hash__(self):
+        return hash((self.name, self.creation_date))
+
+    def __repr__(self):
+        return "name: {}".format(self.name)
 
 class Spec:
     """ FCP root node. Holds all Devices, Messages, Signals, Logs, Configs,
@@ -147,8 +237,9 @@ class Spec:
         self.devices = {}
         self.common = Common()
         self.logs = {}
+        self.enums = {}
         self.version = "0.2"
-
+    
     def add_device(self, device: "Device") -> bool:
         """ Add a Device to Spec. 
 
@@ -165,6 +256,23 @@ class Spec:
 
         return True
 
+    def add_enum(self, enum: "Enum") -> bool:
+        """ Add a Enum to Spec. 
+
+        :param enum: Enum to be added
+        :return: Operation success status: True - Success, False - Failure
+        """
+
+        if enum == None:
+            return False
+
+        if enum.name in self.enums.keys():
+            return False
+
+        self.enums[enum.name] = log
+
+        return True
+
     def add_log(self, log: "Log") -> bool:
         """ Add a Log to Spec. 
 
@@ -181,7 +289,6 @@ class Spec:
         self.logs[log.name] = log
 
         return True
-
     def get_device(self, name: str) -> Optional["Device"]:
         """ Get a Device from Spec by its name.
 
@@ -215,6 +322,8 @@ class Spec:
             self.rm_cmd(node)
         elif type(node) == Log:
             self.rm_log(node)
+        elif type(node) == Enum:
+            self.rm_enum(node)
 
     def rm_device(self, device: "Device") -> None:
         """ Remove a Device from Spec.
@@ -304,13 +413,27 @@ class Spec:
         for name in logs:
             del self.logs[name]
 
+    def rm_enum(self, enum):
+        """ Remove a Enum from Spec.
+
+        :param log: Enum to be removed.
+        """
+        enums = []
+
+        for l in self.enums.values():
+            if enum is l:
+                enums.append(l.name)
+
+        for name in enums:
+            del self.enums[name]
+
     def compile(self) -> Dict[str, Any]:
         """ Transform python class node to its dictionary representation.
 
         :return: A dictionary containing the node parameters
         """
 
-        d = {"devices": {}, "logs": {}}
+        d = {"devices": {}, "logs": {}, "enums": {}}
 
         for dev_k, dev_v in self.devices.items():
             d["devices"][dev_k] = dev_v.compile()
@@ -318,6 +441,9 @@ class Spec:
         for log_k, log_v in self.logs.items():
             d["logs"][log_k] = log_v.compile()
 
+        for enum_k, enum_v in self.enums.items():
+            d["enums"][enum_k] = enum_v.compile()
+
         d["common"] = self.common.compile()
 
         d["version"] = self.version
@@ -333,16 +459,21 @@ class Spec:
         self.devices = {}
         self.logs = {}
         self.common.decompile(d["common"])
-
-        for k, v in d["devices"].items():
+        
+        for k, v in handle_key_not_found(d, "devices"):
             dev = Device()
             dev.decompile(v)
             self.devices[k] = dev
-
-        for k, v in d["logs"].items():
+        
+        for k, v in handle_key_not_found(d, "logs"):
             log = Log()
             log.decompile(v)
             self.logs[k] = log
+       
+        for k, v in handle_key_not_found(d, "enums"):
+            enum = Enum()
+            enum.decompile(v)
+            self.enums[k] = enum
 
         self.versionn = d["version"]
 
@@ -351,6 +482,7 @@ class Spec:
 
         normalize(self.devices)
         normalize(self.logs)
+        normalize(self.enums)
 
         for key, dev in self.devices.items():
             dev.normalize()
@@ -399,6 +531,7 @@ class Signal:
         byte_order: str = "little_endian",
         mux: str = "",
         mux_count: int = 0,
+        alias: str = "",
     ):
 
         self.name = name
@@ -414,6 +547,7 @@ class Signal:
         self.byte_order = byte_order
         self.mux = mux
         self.mux_count = mux_count
+        self.alias = alias
 
         self.creation_date = datetime.datetime.now()
 
@@ -495,6 +629,12 @@ class Signal:
         except Exception as e:
             return
 
+    def set_alias(self, alias: str) -> None:
+        try:
+            self.alias = alias
+        except Exception as e:
+            return
+
     def get_name(self) -> str:
         return self.name
 
@@ -534,6 +674,9 @@ class Signal:
     def get_mux_count(self) -> int:
         return self.mux_count
 
+    def get_alias(self) -> str:
+        return self.alias
+
     def compile(self) -> Dict[str, Any]:
         """ Transform python class node to its dictionary representation.
 
@@ -555,25 +698,23 @@ class Signal:
         return hash((self.name, self.start, self.length, self.creation_date))
 
     def __repr__(self):
-        return (
-            "{"
-            + f("""
-    name: {self.name},
-    start: {self.start}, 
-    length: {self.length}, 
-    scale: {self.scale}, 
-    offset: {self.offset}, 
-    unit: {self.unit}, 
-    comment: {self.comment}, 
-    min: {self.min_value}, 
-    max: {self.max_value}, 
-    type: {self.type}, 
-    byte_order: {self.byte_order}, 
-    mux: {self.mux}, 
-    mux_count: {self.mux_count}
-""")
-            + "}"
-        )
+        return """
+{
+    name: {},
+    start: {}, 
+    length: {}, 
+    scale: {}, 
+    offset: {}, 
+    unit: {}, 
+    comment: {}, 
+    min: {}, 
+    max: {}, 
+    type: {}, 
+    byte_order: {}, 
+    mux: {}, 
+    mux_count: {}
+}
+""".format(self.name, self.start, self.length, self.scale, self.offset, self.unit, self.comment, self.min_value, self.max_value, self.type, self.byte_order, self.mux,self.mux_count)
 
 
 class Message:
@@ -594,6 +735,7 @@ class Message:
         dlc: int = 8,
         signals: Dict[str, Signal] = None,
         frequency: int = 0,
+        description: str = "",
     ):
 
         self.name = name
@@ -601,6 +743,7 @@ class Message:
         self.dlc = dlc
         self.signals = {} if signals == None else signals
         self.frequency = frequency
+        self.description = description
 
         self.creation_date = datetime.datetime.now()
 
@@ -628,6 +771,12 @@ class Message:
         except Exception as e:
             return
 
+    def set_description(self, description: str) -> None:
+        try:
+            self.description = description
+        except Exception as e:
+            return
+
     def get_name(self) -> str:
         return self.name
 
@@ -640,6 +789,9 @@ class Message:
     def get_frequency(self) -> int:
         return self.frequency
 
+    def get_description(self) -> str:
+        return self.description
+
     def compile(self) -> Dict[str, Any]:
         """ Transform python class node to its dictionary representation.
 
@@ -865,7 +1017,7 @@ class Command:
 
         args = {}
         rets = {}
-        print(self.args)
+
         for k, v in self.args.items():
             args[k] = v.compile()
 
